import { MatIconRegistry } from '@angular/material/icon';
import { DomSanitizer } from '@angular/platform-browser';
import * as _ from 'lodash';

/**
 * a utility to load all needed svg resources to the app for mat-icon to use
 *
 * @param ir a MatIconRegistry instance to use external svg resources for mat-icon use
 * @param ds a DomSanitizer instance to bypass security and return a url
 */
export const loadSvgResources = (ir: MatIconRegistry, ds: DomSanitizer) => {
  const imgDir = 'assets/img';
  const avatarDir = `${imgDir}/avatar`;
  const sidebarDir = `${imgDir}/sidebar`;
  const iconDir = `${imgDir}/icons`;
  const dayDir = `${imgDir}/days`;
  ir.addSvgIconSetInNamespace(
    'avatars',
    ds.bypassSecurityTrustResourceUrl(`${avatarDir}/avatars.svg`)
  )
    .addSvgIcon(
      'unassigned',
      ds.bypassSecurityTrustResourceUrl(`${avatarDir}/unassigned.svg`)
    )
    .addSvgIcon(
      'project',
      ds.bypassSecurityTrustResourceUrl(`${sidebarDir}/project.svg`)
    )
    .addSvgIcon(
      'projects',
      ds.bypassSecurityTrustResourceUrl(`${sidebarDir}/projects.svg`)
    )
    .addSvgIcon(
      'month',
      ds.bypassSecurityTrustResourceUrl(`${sidebarDir}/month.svg`)
    )
    .addSvgIcon(
      'week',
      ds.bypassSecurityTrustResourceUrl(`${sidebarDir}/week.svg`)
    )
    .addSvgIcon(
      'day',
      ds.bypassSecurityTrustResourceUrl(`${sidebarDir}/day.svg`)
    )
    .addSvgIcon(
      'move',
      ds.bypassSecurityTrustResourceUrl(`${iconDir}/move.svg`)
    );
  const days = _.range(1, 31);
  days.forEach(day =>
    ir.addSvgIcon(
      `day${day}`,
      ds.bypassSecurityTrustResourceUrl(`${dayDir}/day${day}.svg`)
    )
  );
};
